// Folder selection routine (extended)
//  2004 Jakub Sika. All rights reserved.
//

/////////////////////////////////////////////////////////////////
// @desc
// 
// Demonstrates folder selection Windows API function SHBrowseForFolder().
// Extended version uses callback function and advanced option.

// @tbl
// std Standard

// All symbols are available in shlobj.h
BIF_RETURNONLYFSDIRS		= 1;
BIF_STATUSTEXT			= 4;
BFFM_INITIALIZED		= 1;
BFFM_SELCHANGED			= 2;
// 1024 is a number of WM_USER message
BFFM_SETSTATUSTEXTA		= 1024 + 100;
BFFM_SETSELECTIONA		= 1024 + 102;

/////////////////////////////////////////////////////////////////
// Program entry point
// @std
//
Main()
{
	pszName = MemAlloc(260);
	iId = BrowseFolderEx("Please select your destination (extended version):", 
		"C:\\Windows", BIF_RETURNONLYFSDIRS + BIF_STATUSTEXT, pszName);

	if (MemGetByte(pszName))
		Msg(StrFormat("Item '%s' selected.", pszName));
	else
		Msg("No item selected");

	MemFree(pszName);
	strResult = "";

	if (iId)
	{
		pszPath = MemAlloc(260);

		if (CallProc("SHGetPathFromIDList", "Shell32", iId, pszPath))
			strResult = MemGetStr(pszPath);

		MemFree(pszPath);
	}

	return strResult;
}

/////////////////////////////////////////////////////////////////
// Extended browse folder routine
// @std
//		strTitle: Title displayed for selection
//		strPath: Initial path or empty string
//		iFlags: See example and Win32 documentation
//			BIF_RETURNONLYFSDIRS: Only file system directories
//			BIF_STATUSTEXT: Display status text
//		pszName: Return buffer for item name or NULL
// Return: selected item ID
//
// See Windows API SHGetPathFromIDList() function for more information.
//
BrowseFolderEx(strTitle, strPath, iFlags, pszName)
{
	hShellLib = LoadLib("Shell32");
	pInfo = MemAlloc(64);
	pszPath = MemAlloc(260);
	pszTitle = MemAlloc(260);
	MemSet(pszTitle, strTitle);
	pParam = MemAlloc(8);
	MemSet(pParam, MemAlloc(260));
	MemSet(pParam + 4, iFlags);
	MemSet(MemGetInt(pParam), strPath);
	MemSet(pInfo, VarGetInt("HWND"));	// Use main window handle as a parent
	MemSet(pInfo + 8, pszName);
	MemSet(pInfo + 12, pszTitle);
	MemSet(pInfo + 16, iFlags);
	MemSet(pInfo + 20, CreateProc("BrowseFolderCallback", "i4i4i4i4"));
	MemSet(pInfo + 24, pParam);
	iId = CallProc("SHBrowseForFolder", hShellLib, pInfo);
	MemFree(MemGetInt(pParam));
	MemFree(pParam);
	MemFree(pszTitle);
	MemFree(pszPath);
	DeleteProc(MemGetInt(pInfo + 20));
	MemFree(pInfo);
	FreeLib(hShellLib);

	return iId;
}

/////////////////////////////////////////////////////////////////
// Browse folder callback
// @std
//
// Callback function connected by CreateProc.
// See help and Win32 documentation for more information on callback functions and folders browsing.
//
BrowseFolderCallback()
{
	iMsg = ArrGetInt(VarGetInt("ProcParams"), 1);

	if (iMsg == BFFM_INITIALIZED)
	{
		if (MemGetInt(ArrGetInt(VarGetInt("ProcParams"), 3)))
			CallProc("SendMessageA", VarGetInt("USER32"), ArrGetInt(VarGetInt("ProcParams"), 0), 
				BFFM_SETSELECTIONA, 1, MemGetInt(ArrGetInt(VarGetInt("ProcParams"), 3)));
	}
	else if (iMsg == BFFM_SELCHANGED)
	{
		if (MemGetInt(ArrGetInt(VarGetInt("ProcParams"), 3) + 4) & BIF_STATUSTEXT)
		{
			pszTemp = MemAlloc(260);
			CallProc("SHGetPathFromIDList", "Shell32", ArrGetInt(VarGetInt("ProcParams"), 2), pszTemp);
			CallProc("SendMessageA", VarGetInt("USER32"), ArrGetInt(VarGetInt("ProcParams"), 0), 
				BFFM_SETSTATUSTEXTA, ArrGetInt(VarGetInt("ProcParams"), 3), pszTemp);
			MemFree(pszTemp);
		}
	}

	return 0;
}
