// Custom procedure test
//  2004 Jakub Sika. All rights reserved.
//

/////////////////////////////////////////////////////////////////
// @desc
// 
// Demonstrates creation of custom callback procedure.
// The procedure is callable by CallProc function and also by the operating system.

// @tbl
// std Standard

/////////////////////////////////////////////////////////////////
// Program entry point
// @std
//
Main()
{
	pProc = CreateProc("MyProc", "i4d8s0");	// float MyProc(int, float, char *);
	Result = CallProc(pProc, 187, 11.2, "221");
	DeleteProc(pProc);

	return Result;
}

/////////////////////////////////////////////////////////////////
// Custom callback procedure
// @std
// Return: float sum of the parameters
//
MyProc()
{
	pArray = VarGetInt("ProcParams");
	fSum = 0.0;

	if (pArray)
	{
		for (iIndex = 0, iSize = ArrGetSize(pArray); iIndex < iSize; iIndex ++)
		{
			switch (ArrGetType(pArray, iIndex))
			{
			case 1: // INT
				iValue = ArrGetInt(pArray, iIndex);
				fSum += iValue;
				Msg(StrFormat("Parameter %ld = %ld", iIndex, iValue));

				break;
			case 2: // FLOAT
				fValue = ArrGetFloat(pArray, iIndex);
				fSum += fValue;
				Msg(StrFormat("Parameter %ld = %f", iIndex, fValue));

				break;
			case 3: // STRING
				strValue = ArrGetStr(pArray, iIndex);
				fSum += StrLen(strValue);
				Msg(StrFormat("Parameter %ld = '%s'", iIndex, strValue));

				break;
			default:
				Msg(StrFormat("Parameter %ld ERROR!", iIndex));
			}
		}
	}
	else
		Msg("No params");

	return fSum;
}
