// Main.cpp module for HExec
//  2004 Jakub Sika. All rights reserved.
//
// Syntax: HExec.exe File [Run] [Param1 [Param2 [...]]]
//		Where:
//			File: file or full path: with or without quotation marks
//			Run: -e for statement evaluation or -xFnc for function call (main by default)
//			ParamX: one or more parameters - Name=Text, Name="Text", Name=Float or Name=%Int
//

#include <Windows.h>
#include "HyperScript.h"

#define ERR_CAPT "Error"

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	try
	{
		PLC_InitStd();
		PLC_VarSetString("App", "HExec");
		PLC_VarSetString("Ver", "1.0");
	}
	catch (char *pszError)
	{
		MessageBox(0, pszError, ERR_CAPT, MB_ICONSTOP);
	}

	while (*lpCmdLine == ' ')
		lpCmdLine++;

	if (!(*lpCmdLine))
	{
		const char *pszUsing = "HExec Script.prg [[-e]|[-xFnc]] [Param1=Value2 [Param2=Value2 [...]]]\n"
			"\tScript.prg - file to execute\n"
			"\t-xFnc - execute the function specified (Main by default)\n"
			"\t-e - evaluate one or more statements\n"
			"\tParam{x} - variable name to set\n"
			"\tValue{x} - is specified by following:\n"
			"\t\tString word or float\n"
			"\t\t\"Long string with spaces\"\n"
			"\t\t%Integer followed by percent sign\n"
			"...see the HyperScript documentation for more information\n\n"
			" 2004 - 2005 Jakub Sika. All rights reserved.";
		MessageBox(0, pszUsing, "Using", MB_ICONASTERISK);

		return 0;
	}

	// File examination
	char *pszBuff;
	if (strncmp(lpCmdLine, "-l", 2) == 0)
	{
		try
		{
			PLC_Evaluate(lpCmdLine + 2);
			PLC_DATATYPE *pData = PLC_DT_GetResult();
			pszBuff = (char *)PLC_DT_GetInt(pData);
			PLC_TermStd();
		}
		catch (char *pszError)
		{
			MessageBox(0, pszError, ERR_CAPT, MB_ICONSTOP);
		}

		return (int)pszBuff;
	}
	else if (*lpCmdLine == '"')
		pszBuff = strchr(++lpCmdLine, '"');
	else
		pszBuff = strchr(lpCmdLine, ' ');

	char szFile[MAX_PATH];
	if (pszBuff)
	{
		strncpy(szFile, lpCmdLine, pszBuff - lpCmdLine);
		szFile[pszBuff - lpCmdLine] = 0;
		lpCmdLine = pszBuff + 1;
	}
	else
	{
		strcpy(szFile, lpCmdLine);
		lpCmdLine += strlen(lpCmdLine);
	}

	HANDLE hFile = CreateFile(szFile, GENERIC_READ, 0, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);

	if (hFile == INVALID_HANDLE_VALUE)
	{
		char szMsg[MAX_PATH + 32];
		wsprintf(szMsg, "'%s'\ncould not be open.", szFile);
		MessageBox(0, szMsg, "Error", MB_ICONSTOP);

		return -1;
	}

	unsigned long lRead = GetFileSize(hFile, 0);
	char *pszScript = new char[lRead + 1];

	if (!pszScript)
	{
		CloseHandle(hFile);
		MessageBox(0, "Memory allocation error.", "Error", MB_ICONSTOP);

		return -2;
	}

	if (!ReadFile(hFile, pszScript, lRead, &lRead, 0))
	{
		delete [] pszScript;
		CloseHandle(hFile);

		char szMsg[MAX_PATH + 32];
		wsprintf(szMsg, "'%s'\ncould not be read.", szFile);
		MessageBox(0, szMsg, "Error", MB_ICONSTOP);

		return -3;
	}

	pszScript[lRead] = 0;

	while (*lpCmdLine == ' ')
		lpCmdLine++;

	bool bEvaluate = false;
	char szFunction[64] = "Main";

	// Function or statement call
	if (*lpCmdLine == '-')
	{
		lpCmdLine++;

		if (strcmp(lpCmdLine, "e") == 0 || strncmp(lpCmdLine, "e ", 2) == 0)
		{
			bEvaluate = true;
			lpCmdLine++;
		}
		else if (*lpCmdLine == 'x' && lpCmdLine[1] != ' ')
		{
			pszBuff = strchr(++lpCmdLine, ' ');

			if (pszBuff)
			{
				strncpy(szFunction, lpCmdLine, pszBuff - lpCmdLine);
				szFunction[pszBuff - lpCmdLine] = 0;
				lpCmdLine = pszBuff + 1;
			}
			else
			{
				strcpy(szFunction, lpCmdLine);
				lpCmdLine += strlen(lpCmdLine);
			}
		}
		else
		{
			delete [] pszScript;
			CloseHandle(hFile);

			char szMsg[MAX_PATH + 32];
			wsprintf(szMsg, "Argument: '%s' incorrect!", lpCmdLine);
			MessageBox(0, szMsg, "Error", MB_ICONSTOP);

			return -4;
		}
	}

	while (*lpCmdLine == ' ')
		lpCmdLine++;

	// Parameters examination
	while (*lpCmdLine)
	{
		char szName[32], szValue[256];
		pszBuff = strchr(lpCmdLine, '=');

		if (pszBuff)
		{
			strncpy(szName, lpCmdLine, pszBuff - lpCmdLine);
			szName[pszBuff - lpCmdLine] = 0;
			lpCmdLine = pszBuff + 1;
		}
		else
		{
			strcpy(szName, lpCmdLine);
			lpCmdLine += strlen(lpCmdLine);
		}

		if (*lpCmdLine == '%')
		{
			lpCmdLine++;
			pszBuff = strchr(lpCmdLine, ' ');

			if (pszBuff)
			{
				strncpy(szValue, lpCmdLine, pszBuff - lpCmdLine);
				szValue[pszBuff - lpCmdLine] = 0;
			}
			else
				strcpy(szValue, lpCmdLine);

			PLC_VarSetInt(szName, atol(szValue));
		}
		else if (*lpCmdLine >= '0' && *lpCmdLine <= '9')
		{
			pszBuff = strchr(lpCmdLine, ' ');

			if (pszBuff)
			{
				strncpy(szValue, lpCmdLine, pszBuff - lpCmdLine);
				szValue[pszBuff - lpCmdLine] = 0;
			}
			else
				strcpy(szValue, lpCmdLine);

			PLC_VarSetFloat(szName, (float)atof(szValue));
		}
		else if (*lpCmdLine == '"')
		{
			lpCmdLine++;
			pszBuff = strchr(lpCmdLine, '"');

			if (pszBuff)
			{
				strncpy(szValue, lpCmdLine, pszBuff - lpCmdLine);
				szValue[pszBuff - lpCmdLine] = 0;
			}
			else
				strcpy(szValue, lpCmdLine);

			PLC_VarSetString(szName, szValue);
		}
		else
		{
			pszBuff = strchr(lpCmdLine, ' ');

			if (pszBuff)
			{
				strncpy(szValue, lpCmdLine, pszBuff - lpCmdLine);
				szValue[pszBuff - lpCmdLine] = 0;
			}
			else
				strcpy(szValue, lpCmdLine);

			PLC_VarSetString(szName, szValue);
		}

		if (pszBuff)
			lpCmdLine = pszBuff + 1;
		else
			break;

		while (*lpCmdLine == ' ')
			lpCmdLine++;
	}

	try
	{
		if (bEvaluate)
			PLC_Evaluate(pszScript);
		else
		{
			PLC_Compile(pszScript);
			PLC_Execute(szFunction, 0);
		}
	}
	catch (char *pszError)
	{
		MessageBox(0, pszError, ERR_CAPT, MB_ICONSTOP);
	}

	delete [] pszScript;
	CloseHandle(hFile);

	PLC_DATATYPE *pData = PLC_DT_GetResult();
	lRead = PLC_DT_GetInt(pData);
	PLC_TermStd();

	return lRead;
}
