// HyperDevView.cpp : implementation of the CHyperDevView class
//

#include "stdafx.h"
#include "HyperDev.h"
#include "MainFrm.h"
#include "HyperDevDoc.h"
#include "CntrItem.h"
#include "HyperDevView.h"
#include "HyperScript.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define HD_APP "HyperDev"
#define HD_VER "1.0"

/////////////////////////////////////////////////////////////////////////////
// CHyperDevView

IMPLEMENT_DYNCREATE(CHyperDevView, CRichEditView)

BEGIN_MESSAGE_MAP(CHyperDevView, CRichEditView)
	//{{AFX_MSG_MAP(CHyperDevView)
	ON_WM_DESTROY()
	ON_COMMAND(ID_EDIT_PASTE, OnEditPaste)
	ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE, OnUpdateEditPaste)
	ON_WM_CREATE()
	ON_WM_KEYDOWN()
	ON_WM_LBUTTONDOWN()
	ON_COMMAND(ID_RUN_EXECUTE, OnRunExecute)
	ON_COMMAND(ID_RUN_EVALUATE, OnRunEvaluate)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CRichEditView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CRichEditView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CRichEditView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CHyperDevView construction/destruction

CHyperDevView::CHyperDevView()
{
	// TODO: add construction code here

}

CHyperDevView::~CHyperDevView()
{
}

BOOL CHyperDevView::PreCreateWindow(CREATESTRUCT& cs)
{
	BOOL bResult = CRichEditView::PreCreateWindow(cs);
	cs.style |= ES_MULTILINE;

	return bResult;
}

int CHyperDevView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CRichEditView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	CHARFORMAT cf;
	memset(&cf, 0, sizeof cf);
	cf.cbSize = sizeof CHARFORMAT;
	cf.dwMask = CFM_FACE | CFM_SIZE | CFM_BOLD | CFM_ITALIC | CFM_STRIKEOUT | 
		CFM_UNDERLINE | CFM_COLOR;
	cf.dwEffects = 0;
	cf.yHeight = 220;
	strcpy(cf.szFaceName, "Courier New");
	cf.bCharSet = 0; //DEFAULT_CHARSET;
	cf.crTextColor = RGB(0, 0, 0);
	GetRichEditCtrl().SetDefaultCharFormat(cf);

	return 0;
}

void CHyperDevView::OnInitialUpdate()
{
	CRichEditView::OnInitialUpdate();

	CRichEditCtrl &RichEditCtrl = GetRichEditCtrl();
	//RichEditCtrl.ModifyStyle(0, ES_AUTOHSCROLL | WS_HSCROLL);
	RichEditCtrl.SetTargetDevice(0, 1);
	// Set the printing margins (720 twips = 1/2 inch).
	SetMargins(CRect(720, 720, 720, 720));
}

/////////////////////////////////////////////////////////////////////////////
// CHyperDevView printing

BOOL CHyperDevView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}


void CHyperDevView::OnDestroy()
{
	// Deactivate the item on destruction; this is important
	// when a splitter view is being used.
   CRichEditView::OnDestroy();
   COleClientItem* pActiveItem = GetDocument()->GetInPlaceActiveItem(this);
   if (pActiveItem != NULL && pActiveItem->GetActiveView() == this)
   {
      pActiveItem->Deactivate();
      ASSERT(GetDocument()->GetInPlaceActiveItem(this) == NULL);
   }
}


/////////////////////////////////////////////////////////////////////////////
// CHyperDevView diagnostics

#ifdef _DEBUG
void CHyperDevView::AssertValid() const
{
	CRichEditView::AssertValid();
}

void CHyperDevView::Dump(CDumpContext& dc) const
{
	CRichEditView::Dump(dc);
}

CHyperDevDoc* CHyperDevView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CHyperDevDoc)));
	return (CHyperDevDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CHyperDevView message handlers

void CHyperDevView::UpdateLineNumber()
{
	CRichEditCtrl &RichEditCtrl = GetRichEditCtrl();
	char szMessage[64];
	long lStart, lLine;

	RichEditCtrl.GetSel(lStart, lLine);
	lLine = 0;

	while (lLine < RichEditCtrl.GetLineCount() && lStart >= RichEditCtrl.LineIndex(lLine))
		lLine ++;

	sprintf(szMessage, "Line %ld", lLine);
	((CMainFrame *)g_App.m_pMainWnd)->SetMessageText(szMessage);
}

void CHyperDevView::OnEditPaste() 
{
	GetRichEditCtrl().PasteSpecial(CF_TEXT);
}

void CHyperDevView::OnUpdateEditPaste(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(IsClipboardFormatAvailable(CF_TEXT));
}

void CHyperDevView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	CRichEditView::OnKeyDown(nChar, nRepCnt, nFlags);
	UpdateLineNumber();
}

void CHyperDevView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CRichEditView::OnLButtonDown(nFlags, point);
	UpdateLineNumber();
}

void CHyperDevView::OnRunExecute() 
{
	try
	{
		PLC_DATATYPE *pData;
		CString strScript;
		char szMessage[256];

		PLC_InitStd(AfxGetMainWnd()->m_hWnd);
		PLC_VarSetString("App", HD_APP);
		PLC_VarSetString("Ver", HD_VER);
		GetRichEditCtrl().GetWindowText(strScript);
		PLC_Compile((char *)(const char *)strScript);
		PLC_Execute("Main", 0);
		pData = PLC_DT_GetResult();

		switch (PLC_DT_GetType(pData))
		{
		case PLC_DT_INT:
			sprintf(szMessage, "Result: %i (int)", PLC_DT_GetInt(pData));
			break;
		case PLC_DT_FLOAT:
			sprintf(szMessage, "Result: %f (float)", PLC_DT_GetFloat(pData));
			break;
		case PLC_DT_STRING:
			sprintf(szMessage, "Result: %s (string)", PLC_DT_GetString(pData));
			break;
		}

		((CMainFrame *)g_App.m_pMainWnd)->SetMessageText(szMessage);
	}
	catch (char *pszMessage)
	{
		MessageBox(pszMessage);
	}

	PLC_TermStd();
}

void CHyperDevView::OnRunEvaluate() 
{
	try
	{
		PLC_DATATYPE *pData;
		CString strScript;
		char szMessage[256];

		PLC_InitStd(AfxGetMainWnd()->m_hWnd);
		PLC_VarSetString("App", HD_APP);
		PLC_VarSetString("Ver", HD_VER);
		GetRichEditCtrl().GetWindowText(strScript);
		PLC_Evaluate((char *)(const char *)strScript);
		pData = PLC_DT_GetResult();

		switch (PLC_DT_GetType(pData))
		{
		case PLC_DT_INT:
			sprintf(szMessage, "Result: %i (int)", PLC_DT_GetInt(pData));
			break;
		case PLC_DT_FLOAT:
			sprintf(szMessage, "Result: %f (float)", PLC_DT_GetFloat(pData));
			break;
		case PLC_DT_STRING:
			sprintf(szMessage, "Result: %s (string)", PLC_DT_GetString(pData));
			break;
		}

		((CMainFrame *)g_App.m_pMainWnd)->SetMessageText(szMessage);
	}
	catch (char *pszMessage)
	{
		MessageBox(pszMessage);
	}

	PLC_TermStd();
}
